// src/core.ts
import { base64 } from "rfc4648";

// src/crypto.ts
async function loadCrypto() {
  if (typeof window !== "undefined" && window.crypto || globalThis && globalThis.crypto) {
    const crypto2 = globalThis.crypto || window.crypto;
    return new Promise((resolve) => resolve(crypto2));
  } else {
    const nodeCrypto = await import("crypto");
    return nodeCrypto.webcrypto;
  }
}
var crypto = await loadCrypto();
var crypto_default = crypto;

// src/decrypt-template.html
var decrypt_template_default = '<!DOCTYPE html>\n<html>\n<head>\n    <meta charset="utf-8">\n    <meta name="viewport" content="width=device-width, initial-scale=1">\n    <meta name="robots" content="noindex, nofollow">\n    <title>Protected Page</title>\n    <script type="module">function O(t,e,s){var i;if(s===void 0&&(s={}),!e.codes){e.codes={};for(var r=0;r<e.chars.length;++r)e.codes[e.chars[r]]=r}if(!s.loose&&t.length*e.bits&7)throw new SyntaxError("Invalid padding");for(var n=t.length;t[n-1]==="=";)if(--n,!s.loose&&!((t.length-n)*e.bits&7))throw new SyntaxError("Invalid padding");for(var a=new((i=s.out)!=null?i:Uint8Array)(n*e.bits/8|0),o=0,c=0,f=0,l=0;l<n;++l){var S=e.codes[t[l]];if(S===void 0)throw new SyntaxError("Invalid character "+t[l]);c=c<<e.bits|S,o+=e.bits,o>=8&&(o-=8,a[f++]=255&c>>o)}if(o>=e.bits||255&c<<8-o)throw new SyntaxError("Unexpected end of data");return a}function T(t,e,s){s===void 0&&(s={});for(var i=s,r=i.pad,n=r===void 0?!0:r,a=(1<<e.bits)-1,o="",c=0,f=0,l=0;l<t.length;++l)for(f=f<<8|255&t[l],c+=8;c>e.bits;)c-=e.bits,o+=e.chars[a&f>>c];if(c&&(o+=e.chars[a&f<<e.bits-c]),n)for(;o.length*e.bits&7;)o+="=";return o}var x={chars:"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",bits:6},D={parse:function(e,s){return O(e,x,s)},stringify:function(e,s){return T(e,x,s)}};function h(t){const e=document.querySelector(t);if(e)return e;throw new Error(`No element found with selector: "${t}"`)}const d=h("input"),p=h("header"),M=h("#msg"),m=h("form"),y=h("#load");let g,k,K,N;document.addEventListener("DOMContentLoaded",async()=>{const t=h("pre[data-i]");if(!t.innerText){d.disabled=!0,b("No encrypted payload.");return}N=Number(t.dataset.i);const e=D.parse(t.innerText);if(g=e.slice(0,32),k=e.slice(32,32+16),K=e.slice(32+16),location.hash){const s=new URL(window.location.href);d.value=s.hash.slice(1),s.hash="",history.replaceState(null,"",s.toString())}sessionStorage.k||d.value?await A():(w(y),v(m),p.classList.replace("hidden","flex"),d.focus())});var E,L;const u=((E=window.crypto)==null?void 0:E.subtle)||((L=window.crypto)==null?void 0:L.webkitSubtle);u||(b("Please use a modern browser."),d.disabled=!0);function v(t){t.classList.remove("hidden")}function w(t){t.classList.add("hidden")}function b(t){M.innerText=t,p.classList.add("red")}m.addEventListener("submit",async t=>{t.preventDefault(),await A()});async function P(t){return new Promise(e=>setTimeout(e,t))}async function A(){y.lastElementChild.innerText="Decrypting...",w(p),w(m),v(y),await P(60);try{const t=await F({salt:g,iv:k,ciphertext:K,iterations:N},d.value);document.write(t),document.close()}catch(t){w(y),v(m),p.classList.replace("hidden","flex"),sessionStorage.k?sessionStorage.removeItem("k"):b("Wrong password."),d.value="",d.focus()}}async function C(t,e,s){const i=new TextEncoder,r=await u.importKey("raw",i.encode(e),"PBKDF2",!1,["deriveKey"]);return await u.deriveKey({name:"PBKDF2",salt:t,iterations:s,hash:"SHA-256"},r,{name:"AES-GCM",length:256},!0,["decrypt"])}async function I(t){return u.importKey("jwk",t,"AES-GCM",!0,["decrypt"])}async function F({salt:t,iv:e,ciphertext:s,iterations:i},r){const n=new TextDecoder,a=sessionStorage.k?await I(JSON.parse(sessionStorage.k)):await C(t,r,i),o=new Uint8Array(await u.decrypt({name:"AES-GCM",iv:e},a,s));if(!o)throw"Malformed data";return sessionStorage.k=JSON.stringify(await u.exportKey("jwk",a)),n.decode(o)}</script>\n    <style>*,:before,:after{box-sizing:border-box;border:0}html{line-height:1.5;-webkit-text-size-adjust:100%;-moz-tab-size:4;-o-tab-size:4;tab-size:4;font-family:ui-sans-serif,system-ui,-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Helvetica Neue,Arial,Noto Sans,sans-serif}body{margin:0;line-height:inherit}a{color:inherit;text-decoration:inherit}button,input{font-family:inherit;font-size:100%;line-height:inherit;color:inherit;margin:0;padding:0}button{text-transform:none}button,[type=button],[type=reset],[type=submit]{-webkit-appearance:button}:-moz-focusring{outline:auto}:-moz-ui-invalid{box-shadow:none}p{margin:0}input::-moz-placeholder,input:-ms-input-placeholder,input::placeholder{opacity:1}:disabled{cursor:default}svg{display:block;vertical-align:middle}img,video{max-width:100%;height:auto}[hidden]{display:none}:root{--gray-800: #292524;--gray-700: #433f3b}html,body{color:#fff;font-weight:300}main{background:#000;height:100vh;letter-spacing:.025em;padding:4rem 1rem 1rem}.box{max-width:24rem;width:100%;background:var(--gray-800);padding:1rem;border-radius:.125rem;margin:0 auto;height:170px}header{align-items:center;margin-bottom:1rem;gap:.5rem}#pwd{font-weight:200;border-radius:.125rem;background:var(--gray-800);border:1px solid var(--gray-700);padding:.5rem 1rem;width:100%;color:#fff}#pwd:focus{outline:2px solid transparent;outline-offset:2px}.hidden{display:none!important}.flex{display:flex}#load{display:flex;align-items:center;justify-content:center;height:100%}.red{color:#dc2626}.spinner{pointer-events:none;width:1.5rem;height:1.5rem;border:3px solid transparent;border-color:#fff;border-right-width:2px;border-radius:50%;-webkit-animation:spin .5s linear infinite;animation:spin .5s linear infinite;margin-right:.5rem}#load p:last-child{font-size:1.125rem;line-height:1.75rem}[type=submit]{border-radius:.125rem;color:#000;background:#fff;width:100%;padding:.5rem 0;margin-top:1rem;cursor:pointer}@keyframes spin{to{transform:rotate(360deg)}}#locked{width:1.5rem;height:1.5rem}#msg{font-size:.875rem;line-height:1.25rem}a{position:fixed;bottom:0;right:0;padding:1rem;font-size:.875rem;line-height:1.25rem;color:#fff}a span{font-weight:600}@media (min-width: 475px){main{padding-top:10rem}#msg,a{font-size:1rem;line-height:1.5rem}}</style>\n</head>\n<body>\n    <main>\n        <div class="box">\n            <div id="load">\n                <p class="spinner"></p><p>Loading...</p>\n            </div>\n            <header class="hidden">\n                <svg id="locked" fill="currentColor" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg">\n                    <path fill-rule="evenodd" d="M5 9V7a5 5 0 0110 0v2a2 2 0 012 2v5a2 2 0 01-2 2H5a2 2 0 01-2-2v-5a2 2 0 012-2zm8-2v2H7V7a3 3 0 016 0z" clip-rule="evenodd"></path>\n                </svg>\n                <p id="msg">This page is password protected.</p>\n            </header>\n            <form class="hidden">\n                <input type="password" id="pwd" name="pwd" aria-label="Password" autofocus />\n                <input type="submit" value="Submit">\n            </form>\n        </div>\n        <a href="https://github.com/Greenheart/pagecrypt">Created with <span>PageCrypt</span></a>\n    </main>\n    <!--ENCRYPTED PAYLOAD-->\n</body>\n</html>';

// src/core.ts
async function getEncryptedPayload(content, password, iterations) {
  if (iterations < 2e6) {
    console.warn(
      `[pagecrypt] WARNING: The specified number of password iterations (${iterations}) is not secure. If possible, use at least 2_000_000 or more.`
    );
  }
  const encoder = new TextEncoder();
  const salt = crypto_default.getRandomValues(new Uint8Array(32));
  const baseKey = await crypto_default.subtle.importKey(
    "raw",
    encoder.encode(password),
    "PBKDF2",
    false,
    ["deriveKey"]
  );
  const key = await crypto_default.subtle.deriveKey(
    { name: "PBKDF2", salt, iterations, hash: "SHA-256" },
    baseKey,
    { name: "AES-GCM", length: 256 },
    false,
    ["encrypt"]
  );
  const iv = crypto_default.getRandomValues(new Uint8Array(16));
  const ciphertext = new Uint8Array(
    await crypto_default.subtle.encrypt(
      { name: "AES-GCM", iv },
      key,
      encoder.encode(content)
    )
  );
  const totalLength = salt.length + iv.length + ciphertext.length;
  const mergedData = new Uint8Array(totalLength);
  mergedData.set(salt);
  mergedData.set(iv, salt.length);
  mergedData.set(ciphertext, salt.length + iv.length);
  return base64.stringify(mergedData);
}
async function encryptHTML(inputHTML, password, iterations = 2e6) {
  return decrypt_template_default.replace(
    /<!--ENCRYPTED PAYLOAD-->/,
    `<pre class="hidden" data-i="${iterations.toExponential()}">${await getEncryptedPayload(
      inputHTML,
      password,
      iterations
    )}</pre>`
  );
}
function generatePassword(length = 80, characters = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz") {
  return Array.from({ length }, (_) => getRandomCharacter(characters)).join(
    ""
  );
}
function getRandomCharacter(characters) {
  let randomNumber;
  do {
    randomNumber = crypto_default.getRandomValues(new Uint8Array(1))[0];
  } while (randomNumber >= 256 - 256 % characters.length);
  return characters[randomNumber % characters.length];
}
export {
  encryptHTML,
  generatePassword
};
